@@ -801,7 +801,9 @@ static int strictly_smaller_name(const char * s, const char * t)
 
 /**
  * The name of the link is set to be the GCD of the names of
- * its two endpoints.
+ * its two endpoints. Must be called after each extract_links(),
+ * etc. since that call issues a brand-new set of links into 
+ * parse_info.
  */
 static void compute_link_names(Sentence sent)
 {
@@ -910,6 +912,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 #endif
 	li.null_cost = null_cost(pi);
 	li.link_cost = link_cost(pi);
+	li.corpus_cost = -1.0f;
 	li.and_cost = 0;
 	li.andlist = NULL;
 
@@ -922,7 +925,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 		return li;
 	}
 
-	if (analyze_pass==PP_SECOND_PASS)
+	if (analyze_pass == PP_SECOND_PASS)
 	{
 	  li.andlist = build_andlist(actx, sent);
 	  li.and_cost = li.andlist->cost;
@@ -1034,9 +1037,6 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	Parse_info pi = sent->parse_info;
 	analyze_context_t *actx = sent->analyze_ctxt;
 
-	build_digraph(actx, pi);
-	memset(&li, 0, sizeof(li));
-
 	sublinkage = x_create_sublinkage(pi);
 	postprocessor = sent->dict->postprocessor;
 
@@ -1046,21 +1046,22 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	  copy_full_link(&(sublinkage->link[i]), &(pi->link_array[i]));
 	}
 
-	if (analyze_pass==PP_FIRST_PASS)
+	if (analyze_pass == PP_FIRST_PASS)
 	{
 		post_process_scan_linkage(postprocessor, opts, sent, sublinkage);
 		free_sublinkage(sublinkage);
-		free_digraph(actx, pi);
 		return li;
 	}
 
+	build_digraph(actx, pi);
+
 	/* The code below can be used to generate the "islands" array.
 	 * For this to work, however, you have to call "build_digraph"
 	 * first (as in analyze_fat_linkage). and then "free_digraph".
-	 * For some reason this causes a space leak. XXX !! FIXME!?
 	 */
 	pp = post_process(postprocessor, opts, sent, sublinkage, TRUE);
 
+	memset(&li, 0, sizeof(li));
 	li.N_violations = 0;
 	li.and_cost = 0;
 	li.unused_word_cost = unused_word_cost(sent->parse_info);
@@ -1073,6 +1074,7 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 #endif
 	li.null_cost = null_cost(pi);
 	li.link_cost = link_cost(pi);
+	li.corpus_cost = -1.0f;
 	li.andlist = NULL;
 
 	if (pp == NULL)
@@ -1092,23 +1094,16 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 void extract_thin_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 {
 	int i;
-	Sublinkage *sublinkage;
 	Parse_info pi = sent->parse_info;
 
-	sublinkage = x_create_sublinkage(pi);
-	compute_link_names(sent);
-	for (i=0; i<pi->N_links; i++) {
-		copy_full_link(&sublinkage->link[i],&(pi->link_array[i]));
-	}
-
 	linkage->num_sublinkages = 1;
 	linkage->sublinkage = ex_create_sublinkage(pi);
 
-	for (i=0; i<pi->N_links; ++i) {
-		linkage->sublinkage->link[i] = excopy_link(sublinkage->link[i]);
+	compute_link_names(sent);
+	for (i=0; i<pi->N_links; ++i)
+	{
+		linkage->sublinkage->link[i] = excopy_link(&(pi->link_array[i]));
 	}
-
-	free_sublinkage(sublinkage);
 }
 
 #ifdef DBG
@@ -1189,7 +1184,6 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 
 	analyze_context_t *actx = sent->analyze_ctxt;
 
-	sublinkage = x_create_sublinkage(pi);
 	build_digraph(actx, pi);
 	actx->structure_violation = FALSE;
 	d_root = build_DIS_CON_tree(actx, pi);
@@ -1197,21 +1191,15 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 	if (actx->structure_violation)
 	{
 		compute_link_names(sent);
-		for (i=0; i<pi->N_links; i++)
-		{
-			copy_full_link(&sublinkage->link[i],&(pi->link_array[i]));
-		}
-
 		linkage->num_sublinkages=1;
 		linkage->sublinkage = ex_create_sublinkage(pi);
 
 		/* This will have fat links! */
 		for (i=0; i<pi->N_links; ++i)
 		{
-			linkage->sublinkage->link[i] = excopy_link(sublinkage->link[i]);
+			linkage->sublinkage->link[i] = excopy_link(&(pi->link_array[i]));
 		}
 
-		free_sublinkage(sublinkage);
 		free_digraph(actx, pi);
 		free_DIS_tree(d_root);
 		return;
@@ -1236,6 +1224,7 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 	/* now fill out the sublinkage arrays */
 	compute_link_names(sent);
 
+	sublinkage = x_create_sublinkage(pi);
 	num_sublinkages = 0;
 	for (;;)
 	{
