@@ -28,11 +28,13 @@
 #define _API_STRUCTURESH_
 
 #include "api-types.h"
+#include "structures.h" /* for definition of Link */
 #include "error.h"
 
 typedef enum
 {
-	VDAL, /* sort by Violations, Disjunct cost, And cost, Link cost */
+	VDAL=1, /* Sort by Violations, Disjunct cost, And cost, Link cost */
+	CORPUS, /* Sort by Corpus cost */
 }  Cost_Model_type;
 
 struct Cost_Model_s
@@ -88,6 +90,7 @@ struct Parse_Options_s
 	int display_constituents;     /* if true, output treebank-style constituent structure */
 	int display_bad;              /* if true, bad linkages are displayed */
 	int display_links;            /* if true, a list o' links is printed out */
+	int display_senses;           /* if true, sense candidates are printed out */
 };
 
 typedef struct
@@ -301,7 +304,7 @@ struct Linkage_s
 {
 	int             num_words;  /* number of (tokenized) words */
 	const char *  * word;       /* array of word spellings */
-	Linkage_info    info;       /* index and cost information */
+	Linkage_info*   info;       /* index and cost information */
 	int             num_sublinkages; /* One for thin linkages, bigger for fat */
 	int             current;    /* Allows user to select particular sublinkage */
 	Sublinkage *    sublinkage; /* A parse with conjunctions will have several */
