@@ -11,12 +11,15 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <math.h>
 #include <link-grammar/api.h>
+#include "disjuncts.h"
 #include "error.h"
 #include "preparation.h"
 #ifdef USE_SAT_SOLVER
 #include "sat-solver/sat-encoder.h"
 #endif
+#include "corpus/corpus.h"
 
 /***************************************************************
 *
@@ -44,6 +47,17 @@ static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2)
 	}
 }
 
+#ifdef USE_CORPUS
+static int CORP_compare_parse(Linkage_info * p1, Linkage_info * p2)
+{
+	double diff = p1->corpus_cost - p2->corpus_cost;
+	if (fabs(diff) < 1.0e-5) 
+		return VDAL_compare_parse(p1, p2);
+	if (diff < 0.0f) return -1;
+	return 1;
+}
+#endif
+
 /**
  * Create and initialize a Parse_Options object
  */
@@ -63,7 +77,7 @@ Parse_Options parse_options_create(void)
 	po->null_block = 1;
 	po->islands_ok = FALSE;
 	po->cost_model.compare_fn = &VDAL_compare_parse;
-	po->cost_model.type	   = VDAL;
+	po->cost_model.type = VDAL;
 	po->short_length = 6;
 	po->all_short = FALSE;
 	po->twopass_length = 30;
@@ -84,6 +98,7 @@ Parse_Options parse_options_create(void)
 	po->display_constituents = 0;
 	po->display_bad = FALSE;
 	po->display_links = FALSE;
+	po->display_senses = FALSE;
 
 	return po;
 }
@@ -102,11 +117,24 @@ void parse_options_set_cost_model_type(Parse_Options opts, int cm)
 		opts->cost_model.type = VDAL;
 		opts->cost_model.compare_fn = &VDAL_compare_parse;
 		break;
+	case CORPUS:
+#ifdef USE_CORPUS
+		opts->cost_model.type = CORPUS;
+		opts->cost_model.compare_fn = &CORP_compare_parse;
+#else
+		prt_error("Error: Source code compiled with cost model 'CORPUS' disabled.\n");
+#endif
+		break;
 	default:
 		prt_error("Error: Illegal cost model: %d\n", cm);
 	}
 }
 
+int  parse_options_get_cost_model_type(Parse_Options opts)
+{
+	return opts->cost_model.type;
+}
+
 void parse_options_set_verbosity(Parse_Options opts, int dummy)
 {
 	opts->verbosity = dummy;
@@ -294,6 +322,14 @@ int parse_options_get_display_links(Parse_Options opts) {
 	return opts->display_links;
 }
 
+void parse_options_set_display_senses(Parse_Options opts, int dummy) {
+	opts->display_senses = dummy;
+}
+
+int parse_options_get_display_senses(Parse_Options opts) {
+	return opts->display_senses;
+}
+
 void parse_options_set_display_walls(Parse_Options opts, int dummy) {
 	opts->display_walls = dummy;
 }
@@ -497,6 +533,281 @@ int dictionary_get_max_cost(Dictionary dict)
 	return dict->max_cost;
 }
 
+/***************************************************************
+*
+* Routines for postprocessing
+*
+****************************************************************/
+
+static Linkage_info * linkage_info_new(int num_to_alloc)
+{
+#ifdef USE_CORPUS
+	int i,j;
+#endif
+	Linkage_info *link_info;
+	link_info = (Linkage_info *) xalloc(num_to_alloc * sizeof(Linkage_info));
+#ifdef USE_CORPUS
+	for (i=0; i<num_to_alloc; i++)
+	{
+		Linkage_info *lifo = &link_info[i];
+		for (j=0; j<MAX_SENTENCE; j++)
+		{
+			lifo->disjunct_list_str[j] = NULL;
+		}
+	}
+#endif
+	return link_info;
+}
+
+static void linkage_info_delete(Linkage_info *link_info, int sz)
+{
+#ifdef USE_CORPUS
+	int i,j;
+
+	for (i=0; i<sz; i++)
+	{
+		Linkage_info *lifo = &link_info[i];
+		for (j=0; j<MAX_SENTENCE; j++)
+		{
+			if (lifo->disjunct_list_str[j])
+				free(lifo->disjunct_list_str[j]);
+			lifo->disjunct_list_str[j] = NULL;
+		}
+	}
+#endif
+	xfree(link_info, sz);
+}
+
+static void free_andlists(Sentence sent)
+{
+	int L;
+	Andlist * andlist, * next;
+	for(L=0; L<sent->num_linkages_post_processed; L++) {
+		/* printf("%d ", sent->link_info[L].canonical);  */
+		/* if (sent->link_info[L].canonical==0) continue; */
+		andlist = sent->link_info[L].andlist;
+		while(1) {
+			if(andlist == NULL) break;
+			next = andlist->next;
+			xfree((char *) andlist, sizeof(Andlist));
+			andlist = next;
+		}
+	}
+	/* printf("\n"); */
+}
+
+static void free_post_processing(Sentence sent)
+{
+	if (sent->link_info != NULL) {
+		/* postprocessing must have been done */
+		free_andlists(sent);
+		linkage_info_delete(sent->link_info, sent->num_linkages_alloced);
+		sent->link_info = NULL;
+	}
+}
+
+static void post_process_linkages(Sentence sent, Parse_Options opts)
+{
+	int *indices;
+	int in, block_bottom, block_top;
+	int N_linkages_found, N_linkages_alloced;
+	int N_linkages_post_processed, N_valid_linkages;
+	int overflowed, only_canonical_allowed;
+	Linkage_info *link_info;
+	int canonical;
+#ifdef USE_CORPUS
+	Corpus * corp;
+#endif
+
+	free_post_processing(sent);
+
+	overflowed = build_parse_set(sent, sent->null_count, opts);
+	print_time(opts, "Built parse set");
+
+	if (overflowed)
+	{
+		/* We know that sent->num_linkages_found is bogus, possibly negative */
+		sent->num_linkages_found = opts->linkage_limit;
+		if (1 < opts->verbosity)
+		{
+			err_ctxt ec;
+			ec.sent = sent;
+			err_msg(&ec, Warn, "Warning: Count overflow.\n"
+			  "Considering a random subset of %d of an unknown and large number of linkages\n",
+				opts->linkage_limit);
+		}
+	}
+	N_linkages_found = sent->num_linkages_found;
+
+	if (sent->num_linkages_found == 0)
+	{
+		sent->num_linkages_alloced = 0;
+		sent->num_linkages_post_processed = 0;
+		sent->num_valid_linkages = 0;
+		sent->link_info = NULL;
+		return;
+	}
+
+	if (N_linkages_found > opts->linkage_limit)
+	{
+		N_linkages_alloced = opts->linkage_limit;
+		if (opts->verbosity > 1)
+		{
+			err_ctxt ec;
+			ec.sent = sent;
+			err_msg(&ec, Warn, "Warning: Considering a random subset of %d of %d linkages\n",
+				  N_linkages_alloced, N_linkages_found);
+		}
+	}
+	else
+	{
+		N_linkages_alloced = N_linkages_found;
+	}
+
+	link_info = linkage_info_new(N_linkages_alloced);
+	N_valid_linkages = 0;
+
+	/* Generate an array of linkage indices to examine */
+	indices = (int *) xalloc(N_linkages_alloced * sizeof(int));
+	if (overflowed)
+	{
+		for (in=0; in < N_linkages_alloced; in++)
+		{
+			indices[in] = -(in+1);
+		}
+	}
+	else
+	{
+		sent->rand_state = N_linkages_found + sent->length;
+		for (in=0; in<N_linkages_alloced; in++)
+		{
+			double frac = (double) N_linkages_found;
+			frac /= (double) N_linkages_alloced;
+			block_bottom = (int) (((double) in) * frac);
+			block_top = (int) (((double) (in+1)) * frac);
+			indices[in] = block_bottom + 
+				(rand_r(&sent->rand_state) % (block_top-block_bottom));
+		}
+	}
+
+	only_canonical_allowed = !(overflowed || (N_linkages_found > 2*opts->linkage_limit));
+	/* When we're processing only a small subset of the linkages,
+	 * don't worry about restricting the set we consider to be
+	 * canonical ones.  In the extreme case where we are only
+	 * generating 1 in a million linkages, it's very unlikely
+	 * that we'll hit two symmetric variants of the same linkage
+	 * anyway.
+	 */
+	/* (optional) first pass: just visit the linkages */
+	/* The purpose of these two passes is to make the post-processing
+	 * more efficient.  Because (hopefully) by the time you do the
+	 * real work in the 2nd pass you've pruned the relevant rule set
+	 * in the first pass.
+	 */
+	if (sent->length >= opts->twopass_length)
+	{
+		for (in=0; (in < N_linkages_alloced) &&
+				   (!resources_exhausted(opts->resources)); in++)
+		{
+			extract_links(indices[in], sent->null_count, sent->parse_info);
+			if (set_has_fat_down(sent))
+			{
+				if (only_canonical_allowed && !is_canonical_linkage(sent)) continue;
+				analyze_fat_linkage(sent, opts, PP_FIRST_PASS);
+			}
+			else
+			{
+				analyze_thin_linkage(sent, opts, PP_FIRST_PASS);
+			}
+		}
+	}
+
+#ifdef USE_CORPUS
+	corp = lg_corpus_new();
+#endif
+
+	/* second pass: actually perform post-processing */
+	N_linkages_post_processed = 0;
+	for (in=0; (in < N_linkages_alloced) &&
+			   (!resources_exhausted(opts->resources)); in++)
+	{
+		Linkage_info *lifo = &link_info[N_linkages_post_processed];
+		extract_links(indices[in], sent->null_count, sent->parse_info);
+		if (set_has_fat_down(sent))
+		{
+			canonical = is_canonical_linkage(sent);
+			if (only_canonical_allowed && !canonical) continue;
+			*lifo = analyze_fat_linkage(sent, opts, PP_SECOND_PASS);
+			lifo->fat = TRUE;
+			lifo->canonical = canonical;
+		}
+		else
+		{
+			*lifo = analyze_thin_linkage(sent, opts, PP_SECOND_PASS);
+			lifo->fat = FALSE;
+			lifo->canonical = TRUE;
+		}
+		if (0 == lifo->N_violations)
+		{
+			N_valid_linkages++;
+		}
+		lifo->index = indices[in];
+#ifdef USE_CORPUS
+		lg_corpus_score(corp, sent, lifo);
+#endif
+		N_linkages_post_processed++;
+	}
+
+#ifdef USE_CORPUS
+	lg_corpus_delete(corp);
+#endif
+
+	print_time(opts, "Postprocessed all linkages");
+	qsort((void *)link_info, N_linkages_post_processed, sizeof(Linkage_info),
+		  (int (*)(const void *, const void *)) opts->cost_model.compare_fn);
+
+	if (!resources_exhausted(opts->resources))
+	{
+		if ((N_linkages_post_processed == 0) &&
+		    (N_linkages_found > 0) &&
+		    (N_linkages_found < opts->linkage_limit))
+		{
+			/* With the current parser, the following sentence will elicit
+			 * this error:
+			 *
+			 * Well, say, Joe, you can be Friar Tuck or Much the miller's
+			 * son, and lam me with a quarter-staff; or I'll be the Sheriff
+			 * of Nottingham and you be Robin Hood a little while and kill
+			 * me.
+			 */
+			err_ctxt ec;
+			ec.sent = sent;
+			err_msg(&ec, Error, "Error: None of the linkages is canonical\n"
+			          "\tN_linkages_post_processed=%d "
+			          "N_linkages_found=%d\n",
+			          N_linkages_post_processed,
+			          N_linkages_found);
+		}
+	}
+
+	if (opts->verbosity > 1)
+	{
+		err_ctxt ec;
+		ec.sent = sent;
+		err_msg(&ec, Info, "Info: %d of %d linkages with no P.P. violations\n",
+				N_valid_linkages, N_linkages_post_processed);
+	}
+
+	print_time(opts, "Sorted all linkages");
+
+	sent->num_linkages_alloced = N_linkages_alloced;
+	sent->num_linkages_post_processed = N_linkages_post_processed;
+	sent->num_valid_linkages = N_valid_linkages;
+	sent->link_info = link_info;
+
+	xfree(indices, N_linkages_alloced * sizeof(int));
+	/*if(N_valid_linkages == 0) free_andlists(sent); */
+}
 
 /***************************************************************
 *
@@ -560,34 +871,6 @@ Sentence sentence_create(char *input_string, Dictionary dict)
 	return sent;
 }
 
-static void free_andlists(Sentence sent)
-{
-	int L;
-	Andlist * andlist, * next;
-	for(L=0; L<sent->num_linkages_post_processed; L++) {
-		/* printf("%d ", sent->link_info[L].canonical);  */
-		/* if (sent->link_info[L].canonical==0) continue; */
-		andlist = sent->link_info[L].andlist;
-		while(1) {
-			if(andlist == NULL) break;
-			next = andlist->next;
-			xfree((char *) andlist, sizeof(Andlist));
-			andlist = next;
-		}
-	}
-	/* printf("\n"); */
-}
-
-static void free_post_processing(Sentence sent)
-{
-	if (sent->link_info != NULL) {
-		/* postprocessing must have been done */
-		free_andlists(sent);
-		xfree((char *) sent->link_info, sent->num_linkages_alloced*sizeof(Linkage_info));
-		sent->link_info = NULL;
-	}
-}
-
 void sentence_delete(Sentence sent)
 {
 	if (!sent) return;
@@ -668,194 +951,6 @@ int sentence_nth_word_has_disjunction(Sentence sent, int i) {
 	return (sent->parse_info->chosen_disjuncts[i] != NULL);
 }
 
-/***************************************************************
-*
-* Routines for postprocessing
-*
-****************************************************************/
-
-
-static void post_process_linkages(Sentence sent, Parse_Options opts)
-{
-	int *indices;
-	int in, block_bottom, block_top;
-	int N_linkages_found, N_linkages_alloced;
-	int N_linkages_post_processed, N_valid_linkages;
-	int overflowed, only_canonical_allowed;
-	double denom;
-	Linkage_info *link_info;
-	int canonical;
-
-	free_post_processing(sent);
-
-	overflowed = build_parse_set(sent, sent->null_count, opts);
-	print_time(opts, "Built parse set");
-
-	if (overflowed) {
-		/* We know that sent->num_linkages_found is bogus, possibly negative */
-		sent->num_linkages_found = opts->linkage_limit;
-		if (opts->verbosity > 1)
-		{
-			err_ctxt ec;
-			ec.sent = sent;
-			err_msg(&ec, Warn, "Warning: Count overflow.\n"
-			  "Considering a random subset of %d of an unknown and large number of linkages\n",
-				opts->linkage_limit);
-		}
-	}
-	N_linkages_found = sent->num_linkages_found;
-
-	if (sent->num_linkages_found == 0) {
-		sent->num_linkages_alloced = 0;
-		sent->num_linkages_post_processed = 0;
-		sent->num_valid_linkages = 0;
-		sent->link_info = NULL;
-		return;
-	}
-
-	if (N_linkages_found > opts->linkage_limit)
-	{
-		N_linkages_alloced = opts->linkage_limit;
-		if (opts->verbosity > 1)
-		{
-			err_ctxt ec;
-			ec.sent = sent;
-			err_msg(&ec, Warn, "Warning: Considering a random subset of %d of %d linkages\n",
-				  N_linkages_alloced, N_linkages_found);
-		}
-	}
-	else N_linkages_alloced = N_linkages_found;
-
-	link_info = (Linkage_info *) xalloc(N_linkages_alloced * sizeof(Linkage_info));
-	N_linkages_post_processed = N_valid_linkages = 0;
-
-	/* Generate an array of linkage indices to examine */
-	indices = (int *) xalloc(N_linkages_alloced * sizeof(int));
-	if (overflowed)
-	{
-		for (in=0; in < N_linkages_alloced; in++)
-		{
-			indices[in] = -(in+1);
-		}
-	}
-	else
-	{
-		sent->rand_state = N_linkages_found + sent->length;
-		for (in=0; in<N_linkages_alloced; in++)
-		{
-			denom = (double) N_linkages_alloced;
-			block_bottom = (int) (((double)in*(double) N_linkages_found)/denom);
-			block_top = (int) (((double)(in+1)*(double)N_linkages_found)/denom);
-			indices[in] = block_bottom + (rand_r(&sent->rand_state) % (block_top-block_bottom));
-		}
-	}
-
-	only_canonical_allowed = (!(overflowed || (N_linkages_found > 2*opts->linkage_limit)));
-	/* When we're processing only a small subset of the linkages,
-	 * don't worry about restricting the set we consider to be
-	 * canonical ones.  In the extreme case where we are only
-	 * generating 1 in a million linkages, it's very unlikely
-	 * that we'll hit two symmetric variants of the same linkage
-	 * anyway.
-	 */
-	/* (optional) first pass: just visit the linkages */
-	/* The purpose of these two passes is to make the post-processing
-	 * more efficient.  Because (hopefully) by the time you do the
-	 * real work in the 2nd pass you've pruned the relevant rule set
-	 * in the first pass.
-	 */
-	if (sent->length >= opts->twopass_length)
-	{
-		for (in=0; (in < N_linkages_alloced) &&
-				   (!resources_exhausted(opts->resources)); in++)
-		{
-			extract_links(indices[in], sent->null_count, sent->parse_info);
-			if (set_has_fat_down(sent))
-			{
-				if (only_canonical_allowed && !is_canonical_linkage(sent)) continue;
-				analyze_fat_linkage(sent, opts, PP_FIRST_PASS);
-			}
-			else
-			{
-				analyze_thin_linkage(sent, opts, PP_FIRST_PASS);
-			}
-		}
-	}
-
-	/* second pass: actually perform post-processing */
-	for (in=0; (in < N_linkages_alloced) &&
-			   (!resources_exhausted(opts->resources)); in++)
-	{
-		extract_links(indices[in], sent->null_count, sent->parse_info);
-		if (set_has_fat_down(sent))
-		{
-			canonical = is_canonical_linkage(sent);
-			if (only_canonical_allowed && !canonical) continue;
-			link_info[N_linkages_post_processed] =
-				analyze_fat_linkage(sent, opts, PP_SECOND_PASS);
-			link_info[N_linkages_post_processed].fat = TRUE;
-			link_info[N_linkages_post_processed].canonical = canonical;
-		}
-		else
-		{
-			link_info[N_linkages_post_processed] =
-				analyze_thin_linkage(sent, opts, PP_SECOND_PASS);
-			link_info[N_linkages_post_processed].fat = FALSE;
-			link_info[N_linkages_post_processed].canonical = TRUE;
-		}
-		if (link_info[N_linkages_post_processed].N_violations==0)
-			N_valid_linkages++;
-		link_info[N_linkages_post_processed].index = indices[in];
-		N_linkages_post_processed++;
-	}
-
-	print_time(opts, "Postprocessed all linkages");
-	qsort((void *)link_info, N_linkages_post_processed, sizeof(Linkage_info),
-		  (int (*)(const void *, const void *)) opts->cost_model.compare_fn);
-
-	if (!resources_exhausted(opts->resources))
-	{
-		if ((N_linkages_post_processed == 0) &&
-		    (N_linkages_found > 0) &&
-		    (N_linkages_found < opts->linkage_limit))
-		{
-			/* With the current parser, the following sentence will elicit
-			 * this error:
-			 *
-			 * Well, say, Joe, you can be Friar Tuck or Much the miller's
-			 * son, and lam me with a quarter-staff; or I'll be the Sheriff
-			 * of Nottingham and you be Robin Hood a little while and kill
-			 * me.
-			 */
-			err_ctxt ec;
-			ec.sent = sent;
-			err_msg(&ec, Error, "Error: None of the linkages is canonical\n"
-			          "\tN_linkages_post_processed=%d "
-			          "N_linkages_found=%d\n",
-			          N_linkages_post_processed,
-			          N_linkages_found);
-		}
-	}
-
-	if (opts->verbosity > 1)
-	{
-		err_ctxt ec;
-		ec.sent = sent;
-		err_msg(&ec, Info, "Info: %d of %d linkages with no P.P. violations\n",
-				N_valid_linkages, N_linkages_post_processed);
-	}
-
-	print_time(opts, "Sorted all linkages");
-
-	sent->num_linkages_alloced = N_linkages_alloced;
-	sent->num_linkages_post_processed = N_linkages_post_processed;
-	sent->num_valid_linkages = N_valid_linkages;
-	sent->link_info = link_info;
-
-	xfree(indices, N_linkages_alloced * sizeof(int));
-	/*if(N_valid_linkages == 0) free_andlists(sent); */
-}
-
 int sentence_parse(Sentence sent, Parse_Options opts)
 {
 	int nl;
@@ -936,7 +1031,7 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 	linkage->unionized = FALSE;
 	linkage->sent = sent;
 	linkage->opts = opts;
-	linkage->info = sent->link_info[k];
+	linkage->info = &sent->link_info[k];
 	linkage->dis_con_tree = NULL;
 
 	extract_links(sent->link_info[k].index, sent->null_count, sent->parse_info);
@@ -1240,6 +1335,15 @@ Sentence linkage_get_sentence(Linkage linkage)
 	return linkage->sent;
 }
 
+const char * linkage_get_disjunct(Linkage linkage, int w)
+{
+	if (NULL == linkage->info->disjunct_list_str)
+	{
+		lg_compute_disjunct_strings(linkage->sent, linkage->info);
+	}
+	return linkage->info->disjunct_list_str[w];
+}
+
 const char * linkage_get_word(Linkage linkage, int w)
 {
 	return linkage->word[w];
@@ -1247,38 +1351,43 @@ const char * linkage_get_word(Linkage linkage, int w)
 
 int linkage_unused_word_cost(Linkage linkage)
 {
-	return linkage->info.unused_word_cost;
+	return linkage->info->unused_word_cost;
 }
 
 int linkage_disjunct_cost(Linkage linkage)
 {
-	return linkage->info.disjunct_cost;
+	return linkage->info->disjunct_cost;
 }
 
 int linkage_and_cost(Linkage linkage)
 {
-	return linkage->info.and_cost;
+	return linkage->info->and_cost;
 }
 
 int linkage_link_cost(Linkage linkage)
 {
-	return linkage->info.link_cost;
+	return linkage->info->link_cost;
+}
+
+double linkage_corpus_cost(Linkage linkage)
+{
+	return linkage->info->corpus_cost;
 }
 
 int linkage_get_link_num_domains(Linkage linkage, int index)
 {
-	PP_info pp_info;
+	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return -1;
-	pp_info = linkage->sublinkage[linkage->current].pp_info[index];
-	return pp_info.num_domains;
+	pp_info = &linkage->sublinkage[linkage->current].pp_info[index];
+	return pp_info->num_domains;
 }
 
-char ** linkage_get_link_domain_names(Linkage linkage, int index)
+const char ** linkage_get_link_domain_names(Linkage linkage, int index)
 {
-	PP_info pp_info;
+	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return NULL;
-	pp_info = linkage->sublinkage[linkage->current].pp_info[index];
-	return pp_info.domain_name;
+	pp_info = &linkage->sublinkage[linkage->current].pp_info[index];
+	return pp_info->domain_name;
 }
 
 const char * linkage_get_violation_name(Linkage linkage)
@@ -1288,17 +1397,17 @@ const char * linkage_get_violation_name(Linkage linkage)
 
 int linkage_is_canonical(Linkage linkage)
 {
-	return linkage->info.canonical;
+	return linkage->info->canonical;
 }
 
 int linkage_is_improper(Linkage linkage)
 {
-	return linkage->info.improper_fat_linkage;
+	return linkage->info->improper_fat_linkage;
 }
 
 int linkage_has_inconsistent_domains(Linkage linkage)
 {
-	return linkage->info.inconsistent_domains;
+	return linkage->info->inconsistent_domains;
 }
 
 void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
@@ -1335,7 +1444,7 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 			subl->violation = NULL;
 		}
 
-		if (linkage->info.improper_fat_linkage)
+		if (linkage->info->improper_fat_linkage)
 		{
 			pp = NULL;
 		}
