@@ -28,6 +28,7 @@ static struct
 	int panic_mode;
 	int allow_null;
 	int echo_on;
+	int cost_model;
 	int screen_width;
 	int display_on;
 	int display_constituents;
@@ -37,6 +38,7 @@ static struct
 	int display_links;
 	int display_walls;
 	int display_union;
+	int display_senses;
 } local;
 
 typedef struct
@@ -49,27 +51,29 @@ typedef struct
 
 static Switch default_switches[] =
 {
-	{"verbosity",	0, "Level of detail in output",		&local.verbosity},
-	{"timeout",	  0, "Abort parsing after this long",	&local.timeout},
-	{"memory",	   0, "Max memory allowed",			   &local.memory},
-	{"limit",		0, "The maximum linkages processed",   &local.linkage_limit},
-	{"null-block",   0, "Size of blocks with null cost 1",  &local.null_block},
-	{"islands-ok",   1, "Use of null-linked islands",	   &local.islands_ok},
-	{"short",		0, "Max length of short links",		&local.short_length},
-	{"batch",		1, "Batch mode",					   &local.batch_mode},
-	{"panic",		1, "Use of \"panic mode\"",			&local.panic_mode},
-	{"null",		 1, "Null links",					   &local.allow_null},
-	{"width",		0, "The width of the display",		 &local.screen_width},
-	{"echo",		 1, "Echoing of input sentence",		&local.echo_on},
-	{"graphics",	 1, "Graphical display of linkage",	 &local.display_on},
-	{"postscript",   1, "Generate postscript output",	   &local.display_postscript},
-	{"constituents", 0, "Generate constituent output",	  &local.display_constituents},
-	{"max-length",   0, "Maximum sentence length",		  &local.max_sentence_length},
-	{"bad",		  1, "Display of bad linkages",		  &local.display_bad},
-	{"links",		1, "Showing of complete link data",	&local.display_links},
-	{"walls",		1, "Showing of wall words",			&local.display_walls},
-	{"union",		1, "Showing of 'union' linkage",	   &local.display_union},
-	{NULL,		   1,  NULL,							  NULL}
+   {"bad",        1, "Display of bad linkages",         &local.display_bad},
+   {"batch",      1, "Batch mode",                      &local.batch_mode},
+   {"constituents", 0, "Generate constituent output",   &local.display_constituents},
+   {"cost",       0, "Cost model used for ranking",     &local.cost_model},
+   {"echo",       1, "Echoing of input sentence",       &local.echo_on},
+   {"graphics",   1, "Graphical display of linkage",    &local.display_on},
+   {"islands-ok", 1, "Use of null-linked islands",      &local.islands_ok},
+   {"limit",      0, "The maximum linkages processed",  &local.linkage_limit},
+   {"links",      1, "Showing of complete link data",   &local.display_links},
+   {"max-length", 0, "Maximum sentence length",         &local.max_sentence_length},
+   {"memory",     0, "Max memory allowed",              &local.memory},
+   {"null",       1, "Null links",                      &local.allow_null},
+   {"null-block", 0, "Size of blocks with null cost 1", &local.null_block},
+   {"panic",      1, "Use of \"panic mode\"",           &local.panic_mode},
+   {"postscript", 1, "Generate postscript output",      &local.display_postscript},
+   {"senses",     1, "Showing of word senses",          &local.display_senses},
+   {"short",      0, "Max length of short links",       &local.short_length},
+   {"timeout",    0, "Abort parsing after this long",   &local.timeout},
+   {"union",      1, "Showing of 'union' linkage",      &local.display_union},
+   {"verbosity",  0, "Level of detail in output",       &local.verbosity},
+   {"walls",      1, "Showing of wall words",           &local.display_walls},
+   {"width",      0, "The width of the display",        &local.screen_width},
+   {NULL,         1,  NULL,                             NULL}
 };
 
 struct {const char * s; const char * str;} user_command[] =
@@ -318,6 +322,7 @@ static void put_opts_in_local_vars(Parse_Options opts)
 	local.null_block = parse_options_get_null_block(opts);
 	local.islands_ok = parse_options_get_islands_ok(opts);
 	local.short_length = parse_options_get_short_length(opts);
+	local.cost_model = parse_options_get_cost_model_type(opts);
 	local.echo_on = parse_options_get_echo_on(opts);
 	local.batch_mode = parse_options_get_batch_mode(opts);
 	local.panic_mode = parse_options_get_panic_mode(opts);
@@ -330,6 +335,7 @@ static void put_opts_in_local_vars(Parse_Options opts)
 	local.max_sentence_length = parse_options_get_max_sentence_length(opts);
 	local.display_bad = parse_options_get_display_bad(opts);
 	local.display_links = parse_options_get_display_links(opts);
+	local.display_senses = parse_options_get_display_senses(opts);
 	local.display_walls = parse_options_get_display_walls(opts);
 	local.display_union = parse_options_get_display_union(opts);
 }
@@ -344,6 +350,7 @@ static void put_local_vars_in_opts(Parse_Options opts)
 	parse_options_set_islands_ok(opts, local.islands_ok);
 	parse_options_set_short_length(opts, local.short_length);
 	parse_options_set_echo_on(opts, local.echo_on);
+	parse_options_set_cost_model_type(opts, local.cost_model);
 	parse_options_set_batch_mode(opts, local.batch_mode);
 	parse_options_set_panic_mode(opts, local.panic_mode);
 	parse_options_set_screen_width(opts, local.screen_width);
@@ -355,6 +362,7 @@ static void put_local_vars_in_opts(Parse_Options opts)
 	parse_options_set_max_sentence_length(opts, local.max_sentence_length);
 	parse_options_set_display_bad(opts, local.display_bad);
 	parse_options_set_display_links(opts, local.display_links);
+	parse_options_set_display_senses(opts, local.display_senses);
 	parse_options_set_display_walls(opts, local.display_walls);
 	parse_options_set_display_union(opts, local.display_union);
 }
