@@ -114,6 +114,8 @@ link_public_api(int)
      parse_options_get_max_parse_time(Parse_Options opts);
 link_public_api(void)
      parse_options_set_cost_model_type(Parse_Options opts, int cm);
+link_public_api(int)
+     parse_options_get_cost_model_type(Parse_Options opts);
 link_public_api(int)
      parse_options_timer_expired(Parse_Options opts);
 link_public_api(int)
@@ -178,6 +180,10 @@ link_public_api(void)
      parse_options_set_display_links(Parse_Options opts, int val);
 link_public_api(int)
      parse_options_get_display_links(Parse_Options opts);
+link_public_api(void)
+     parse_options_set_display_senses(Parse_Options opts, int val);
+link_public_api(int)
+     parse_options_get_display_senses(Parse_Options opts);
 link_public_api(void)
      parse_options_set_display_union(Parse_Options opts, int val);
 link_public_api(int)
@@ -269,16 +275,22 @@ link_public_api(const char *)
      linkage_get_link_rlabel(Linkage linkage, int index);
 link_public_api(int)
      linkage_get_link_num_domains(Linkage linkage, int index);
-link_public_api(char **)
+link_public_api(const char **)
      linkage_get_link_domain_names(Linkage linkage, int index);
 link_public_api(const char **)
      linkage_get_words(Linkage linkage);
+link_public_api(const char *)
+     linkage_get_disjunct(Linkage linkage, int w);
 link_public_api(const char *)
      linkage_get_word(Linkage linkage, int w);
 link_public_api(char *)
      linkage_print_links_and_domains(Linkage linkage);
 link_public_api(void)
      linkage_free_links_and_domains(char *str);
+link_public_api(char *)
+     linkage_print_senses(Linkage linkage);
+link_public_api(void)
+     linkage_free_senses(char *str);
 link_public_api(char *)
      linkage_print_constituent_tree(Linkage linkage, int mode);
 link_public_api(void)
@@ -301,6 +313,8 @@ link_public_api(int)
      linkage_and_cost(Linkage linkage);
 link_public_api(int)
      linkage_link_cost(Linkage linkage);
+link_public_api(double)
+     linkage_corpus_cost(Linkage linkage);
 link_public_api(int)
      linkage_is_canonical(Linkage linkage);
 link_public_api(int)
