@@ -14,6 +14,7 @@
 
 #include <stdarg.h>
 #include <link-grammar/api.h>
+#include "corpus/corpus.h"
 
 const char * trailer(int mode);
 const char * header(int mode);
@@ -133,6 +134,46 @@ char * linkage_print_links_and_domains(Linkage linkage)
 	return links_string; 
 }
 
+char * linkage_print_senses(Linkage linkage)
+{
+	String * s = string_new();
+	char * sense_string;
+#ifdef USE_CORPUS
+	Corpus * corp;
+	const char * infword;
+	Sentence sent = linkage->sent;
+	int nwords = sent->length;
+	Linkage_info *lifo = linkage->info;
+	int w;
+
+	corp = lg_corpus_new();
+
+	/* Decrement nwords, so as to ignore the RIGHT-WALL */
+	nwords --;
+
+	/* Loop over each word in the sentence (skipping LEFT-WALL, which is
+	 * word 0. */
+	for (w=1; w<nwords; w++)
+	{
+		/* If the word is not inflected, then sent->word[w].d is NULL */
+		if (sent->word[w].d)
+			infword = sent->word[w].d->string;
+		else
+			infword = sent->word[w].string;
+
+		lg_corpus_senses(corp, infword, lifo->disjunct_list_str[w]);
+	}
+	lg_corpus_delete(corp);
+
+	append_string(s, " ");
+#else
+	append_string(s, "Corpus statstics is not enabled in this version\n");
+#endif
+	sense_string = string_copy(s);
+	string_delete(s);
+	return sense_string;
+}
+
 /**
  */
 static char * build_linkage_postscript_string(Linkage linkage, ps_ctxt_t *pctx)
@@ -551,12 +592,17 @@ char * linkage_print_diagram(Linkage linkage)
 
 void linkage_free_diagram(char * s)
 {
-		exfree(s, strlen(s)+1);
+	exfree(s, strlen(s)+1);
 }
 
 void linkage_free_links_and_domains(char * s)
 {
-		exfree(s, strlen(s)+1);
+	exfree(s, strlen(s)+1);
+}
+
+void linkage_free_senses(char * s)
+{
+	exfree(s, strlen(s)+1);
 }
 
 char * linkage_print_postscript(Linkage linkage, int mode)
