@@ -311,9 +311,10 @@ struct Andlist_struct
     int cost;
 };
 
-/* This is for building the graphs of links in post-processing and          */
-/* fat link extraction.                                                     */
-
+/** 
+ * This is for building the graphs of links in post-processing and 
+ * fat link extraction.
+ */
 struct Linkage_info_struct
 {
     int index;
@@ -321,9 +322,12 @@ struct Linkage_info_struct
     char canonical;
     char improper_fat_linkage;
     char inconsistent_domains;
-    short N_violations, null_cost, unused_word_cost, disjunct_cost, and_cost, link_cost;
+    short N_violations;
+    short null_cost, unused_word_cost, disjunct_cost, and_cost, link_cost;
+    double corpus_cost;
     Andlist * andlist;
     int island[MAX_SENTENCE];
+    char *disjunct_list_str[MAX_SENTENCE];
 };
 
 struct List_o_links_struct
